@@ -130,6 +130,17 @@ struct StringConfigData
     bool        changed;
 };
 
+struct ColorConfigData
+{
+    const char *key;
+    String      value;
+    const char *label;
+    const char *title;
+    const char *tooltip;
+    GtkWidget  *widget;
+    bool        changed;
+};
+
 struct KeyboardConfigPage
 {
     const char       *label;
@@ -163,6 +174,7 @@ static GtkWidget   * __widget_key_categories_menu = NULL;
 static GtkWidget   * __widget_key_filter          = NULL;
 static GtkWidget   * __widget_key_filter_button   = NULL;
 static GtkWidget   * __widget_key_list_view       = NULL;
+static GtkWidget   * __widget_choose_keys_button  = NULL;
 static GtkTooltips * __widget_tooltips            = NULL;
 
 static BoolConfigData __config_bool_common [] =
@@ -758,6 +770,38 @@ static StringConfigData __config_keyboards_direct_select_candidate [] =
     },
 };
 
+static ColorConfigData __config_color_common [] =
+{
+    {
+        SCIM_PRIME_CONFIG_CANDIDATE_FORM_COLOR,
+        SCIM_PRIME_CONFIG_CANDIDATE_FORM_COLOR_DEFAULT,
+        N_("Co_lor:"),
+        N_("The color of the annotaion text"),
+        N_("The color of the annotaion text in the candidate list."),
+        NULL,
+        false,
+    },
+    {
+        SCIM_PRIME_CONFIG_CANDIDATE_USAGE_COLOR,
+        SCIM_PRIME_CONFIG_CANDIDATE_USAGE_COLOR_DEFAULT,
+        N_("Co_lor:"),
+        N_("The color of the usage text"),
+        N_("The color of the usage text in the candidate list."),
+        NULL,
+        false,
+    },
+    {
+        SCIM_PRIME_CONFIG_CANDIDATE_COMMENT_COLOR,
+        SCIM_PRIME_CONFIG_CANDIDATE_COMMENT_COLOR_DEFAULT,
+        N_("Co_lor:"),
+        N_("The color of the comment text"),
+        N_("The color of the comment text in the candidate list."),
+        NULL,
+        false,
+    },
+};
+static unsigned int __config_color_common_num = sizeof (__config_color_common) / sizeof (ColorConfigData);
+
 static struct KeyboardConfigPage __key_conf_pages[] =
 {
     {N_("Edit keys"),       __config_keyboards_edit},
@@ -783,6 +827,9 @@ static ComboConfigCandidate predict_win_pos[] =
 
 static void on_default_editable_changed       (GtkEditable     *editable,
                                                gpointer         user_data);
+static void on_toggle_button_toggled_set_sensitive
+                                              (GtkToggleButton *togglebutton,
+                                               gpointer         user_data);
 static void on_default_toggle_button_toggled  (GtkToggleButton *togglebutton,
                                                gpointer         user_data);
 #if 0
@@ -791,6 +838,9 @@ static void on_default_key_selection_clicked  (GtkButton       *button,
 #endif
 static void on_default_combo_changed          (GtkEditable     *editable,
                                                gpointer         user_data);
+static void on_default_color_button_set       (GtkColorButton  *colorbutton,
+                                               gpointer         user_data);
+
 static void     on_key_filter_selection_clicked   (GtkButton       *button,
                                                    gpointer         user_data);
 static void     on_key_category_menu_changed      (GtkOptionMenu   *omenu,
@@ -801,6 +851,10 @@ static gboolean on_key_list_view_key_press        (GtkWidget       *widget,
 static gboolean on_key_list_view_button_press     (GtkWidget       *widget,
                                                    GdkEventButton  *event,
                                                    gpointer         user_data);
+static void     on_key_list_selection_changed     (GtkTreeSelection *selection,
+                                                   gpointer          data);
+static void     on_choose_keys_button_clicked     (GtkWidget        *button,
+                                                   gpointer          data);
 static void setup_widget_value ();
 
 
@@ -835,6 +889,21 @@ find_string_config_entry (const char *config_key)
     return NULL;
 }
 
+static ColorConfigData *
+find_color_config_entry (const char *config_key)
+{
+    if (!config_key)
+        return NULL;
+
+    for (unsigned int i = 0; i < __config_color_common_num; i++) {
+        ColorConfigData *entry = &__config_color_common[i];
+        if (entry->key && !strcmp (entry->key, config_key))
+            return entry;
+    }
+
+    return NULL;
+}
+
 static GtkWidget *
 create_check_button (const char *config_key)
 {
@@ -870,6 +939,36 @@ match_key_event (const KeyEventList &list, const KeyEvent &key)
     return false;
 }
 
+static void
+create_entry (StringConfigData *data, GtkTable *table, int i)
+{
+    GtkWidget *label = gtk_label_new (NULL);
+    gtk_label_set_text_with_mnemonic (GTK_LABEL (label), _(data->label));
+    gtk_widget_show (label);
+    gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+    gtk_misc_set_padding (GTK_MISC (label), 4, 0);
+    gtk_table_attach (GTK_TABLE (table), label, 0, 1, i, i+1,
+                      (GtkAttachOptions) (GTK_FILL),
+                      (GtkAttachOptions) (GTK_FILL), 4, 4);
+    (data)->widget = gtk_entry_new ();
+    gtk_label_set_mnemonic_widget (GTK_LABEL (label),
+                                   GTK_WIDGET (data->widget));
+    g_signal_connect ((gpointer) (data)->widget, "changed",
+                      G_CALLBACK (on_default_editable_changed),
+                      data);
+    gtk_widget_show (GTK_WIDGET (data->widget));
+    gtk_table_attach (GTK_TABLE (table), GTK_WIDGET (data->widget),
+                      1, 2, i, i+1,
+                      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+                      (GtkAttachOptions) (GTK_FILL), 4, 4);
+
+    if (!__widget_tooltips)
+        __widget_tooltips = gtk_tooltips_new();
+    if (data->tooltip)
+        gtk_tooltips_set_tip (__widget_tooltips, GTK_WIDGET (data->widget),
+                              _(data->tooltip), NULL);
+}
+
 static GtkWidget *
 create_combo (const char *config_key, gpointer candidates_p,
               GtkWidget *table, gint idx)
@@ -914,31 +1013,42 @@ create_combo (const char *config_key, gpointer candidates_p,
     return entry->widget;
 }
 
-#define APPEND_ENTRY(data, i)                                                  \
-{                                                                              \
-    label = gtk_label_new (NULL);                                              \
-    gtk_label_set_text_with_mnemonic (GTK_LABEL (label), _((data)->label));    \
-    gtk_widget_show (label);                                                   \
-    gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);                       \
-    gtk_misc_set_padding (GTK_MISC (label), 4, 0);                             \
-    gtk_table_attach (GTK_TABLE (table), label, 0, 1, i, i+1,                  \
-                      (GtkAttachOptions) (GTK_FILL),                           \
-                      (GtkAttachOptions) (GTK_FILL), 4, 4);                    \
-    (data)->widget = gtk_entry_new ();                                         \
-    gtk_label_set_mnemonic_widget (GTK_LABEL (label), (data)->widget);         \
-    g_signal_connect ((gpointer) (data)->widget, "changed",                    \
-                      G_CALLBACK (on_default_editable_changed),                \
-                      (data));                                                 \
-    gtk_widget_show ((data)->widget);                                          \
-    gtk_table_attach (GTK_TABLE (table), (data)->widget, 1, 2, i, i+1,         \
-                      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),                \
-                      (GtkAttachOptions) (GTK_FILL), 4, 4);                    \
-                                                                               \
-    if (!__widget_tooltips)                                                    \
-        __widget_tooltips = gtk_tooltips_new();                                \
-    if ((data)->tooltip)                                                       \
-        gtk_tooltips_set_tip (__widget_tooltips, (data)->widget,               \
-                              _((data)->tooltip), NULL);                       \
+static GtkWidget *
+create_color_button (const char *config_key)
+{
+    ColorConfigData *entry = find_color_config_entry (config_key);
+    if (!entry)
+        return NULL;
+
+    GtkWidget *hbox = gtk_hbox_new (FALSE, 0);
+    gtk_widget_show (hbox);
+
+    GtkWidget *label = NULL;
+    if (entry->label) {
+        label = gtk_label_new_with_mnemonic (_(entry->label));
+        gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);
+        gtk_widget_show (label);
+    }
+
+    entry->widget = gtk_color_button_new ();
+    gtk_color_button_set_title (GTK_COLOR_BUTTON (entry->widget), entry->title);
+    gtk_container_set_border_width (GTK_CONTAINER (entry->widget), 4);
+    g_signal_connect (G_OBJECT (entry->widget), "color-set",
+                      G_CALLBACK (on_default_color_button_set),
+                      entry);
+    gtk_box_pack_start (GTK_BOX (hbox), entry->widget, FALSE, FALSE, 2);
+    gtk_widget_show (entry->widget);
+
+    if (label)
+        gtk_label_set_mnemonic_widget (GTK_LABEL (label), entry->widget);
+
+    if (!__widget_tooltips)
+        __widget_tooltips = gtk_tooltips_new();
+    if (entry->tooltip)
+        gtk_tooltips_set_tip (__widget_tooltips, entry->widget,
+                              _(entry->tooltip), NULL);
+
+    return hbox;
 }
 
 static void
@@ -1028,7 +1138,7 @@ key_list_view_popup_key_selection (GtkTreeView *treeview)
 static GtkWidget *
 create_options_page ()
 {
-    GtkWidget *vbox, *table, *label, *widget;
+    GtkWidget *vbox, *table, *widget;
     StringConfigData *entry;
 
     vbox = gtk_vbox_new (FALSE, 0);
@@ -1039,7 +1149,7 @@ create_options_page ()
     gtk_widget_show (table);
 
     entry = find_string_config_entry (SCIM_PRIME_CONFIG_COMMAND);
-    APPEND_ENTRY(entry, 0);
+    create_entry (entry, GTK_TABLE (table), 0);
 
     /* start conversion on inputting comma or period */
     widget = create_check_button (SCIM_PRIME_CONFIG_CONVERT_ON_PERIOD);
@@ -1089,34 +1199,60 @@ create_prediction_page ()
 static GtkWidget *
 create_candidates_window_page ()
 {
-    GtkWidget *vbox, *table, *widget;
+    GtkWidget *vbox, *hbox, *widget, *widget2;
 
     vbox = gtk_vbox_new (FALSE, 0);
     gtk_widget_show (vbox);
 
-    table = gtk_table_new (2, 2, FALSE);
-    gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 4);
-    gtk_widget_show (table);
-
     /* auto register */
     widget = create_check_button (SCIM_PRIME_CONFIG_AUTO_REGISTER);
-    gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, FALSE, 4);
+    gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, FALSE, 2);
 
     /* close candidate window on select */
     widget = create_check_button (SCIM_PRIME_CONFIG_CLOSE_CAND_WIN_ON_SELECT);
-    gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, FALSE, 4);
+    gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, FALSE, 2);
 
     /* show annotation */
+    hbox = gtk_hbox_new (FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 2);
+    gtk_widget_show (hbox);
     widget = create_check_button (SCIM_PRIME_CONFIG_SHOW_ANNOTATION);
-    gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, FALSE, 4);
+    gtk_box_pack_start (GTK_BOX (hbox), widget, TRUE, TRUE, 0);
+    widget2 = widget;
+    /* annotation color */
+    widget = create_color_button (SCIM_PRIME_CONFIG_CANDIDATE_FORM_COLOR);
+    gtk_box_pack_end (GTK_BOX (hbox), widget, FALSE, FALSE, 0);
+    g_signal_connect (G_OBJECT (widget2), "toggled",
+                      G_CALLBACK (on_toggle_button_toggled_set_sensitive),
+                      widget);
 
     /* show usage */
+    hbox = gtk_hbox_new (FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 2);
+    gtk_widget_show (hbox);
     widget = create_check_button (SCIM_PRIME_CONFIG_SHOW_USAGE);
-    gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, FALSE, 4);
+    gtk_box_pack_start (GTK_BOX (hbox), widget, TRUE, TRUE, 0);
+    widget2 = widget;
+    /* usage text color */
+    widget = create_color_button (SCIM_PRIME_CONFIG_CANDIDATE_USAGE_COLOR);
+    gtk_box_pack_start (GTK_BOX (hbox), widget, FALSE, FALSE, 0);
+    g_signal_connect (G_OBJECT (widget2), "toggled",
+                      G_CALLBACK (on_toggle_button_toggled_set_sensitive),
+                      widget);
 
     /* show comment */
+    hbox = gtk_hbox_new (FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 2);
+    gtk_widget_show (hbox);
     widget = create_check_button (SCIM_PRIME_CONFIG_SHOW_COMMENT);
-    gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, FALSE, 4);
+    gtk_box_pack_start (GTK_BOX (hbox), widget, TRUE, TRUE, 0);
+    widget2 = widget;
+    /* comment text color */
+    widget = create_color_button (SCIM_PRIME_CONFIG_CANDIDATE_COMMENT_COLOR);
+    gtk_box_pack_start (GTK_BOX (hbox), widget, FALSE, FALSE, 0);
+    g_signal_connect (G_OBJECT (widget2), "toggled",
+                      G_CALLBACK (on_toggle_button_toggled_set_sensitive),
+                      widget);
 
     return vbox;
 }
@@ -1250,6 +1386,24 @@ create_keyboard_page (void)
                                                        NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
 
+    // for key bind theme
+    hbox = gtk_hbox_new (FALSE, 0);
+    gtk_container_set_border_width (GTK_CONTAINER (hbox), 4);
+    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
+    gtk_widget_show(hbox);
+
+    // edit button
+    button = gtk_button_new_with_mnemonic (_("_Choose keys..."));
+    __widget_choose_keys_button = button;
+    g_signal_connect (G_OBJECT (button), "clicked",
+                      G_CALLBACK (on_choose_keys_button_clicked), treeview);
+    gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 2);
+    gtk_widget_set_sensitive (button, false);
+    gtk_widget_show (button);
+
+    GtkTreeSelection *selection;
+    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
+
     // connect signals
     g_signal_connect (G_OBJECT (omenu), "changed",
                       G_CALLBACK (on_key_category_menu_changed), treeview);
@@ -1257,6 +1411,8 @@ create_keyboard_page (void)
                       G_CALLBACK (on_key_list_view_key_press), NULL);
     g_signal_connect (G_OBJECT (treeview), "button-press-event",
                       G_CALLBACK (on_key_list_view_button_press), NULL);
+    g_signal_connect (G_OBJECT (selection), "changed",
+                      G_CALLBACK (on_key_list_selection_changed), treeview);
 
     return vbox;
 }
@@ -1273,44 +1429,44 @@ create_setup_window ()
         window = notebook;
         gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook), TRUE);
 
-        // Create the 1st page.
+        // Create the common page.
         GtkWidget *page = create_options_page ();
         GtkWidget *label = gtk_label_new (_("Common"));
         gtk_widget_show (label);
         gtk_notebook_append_page (GTK_NOTEBOOK (notebook), page, label);
 
-        // Create the 2nd page.
-        page = create_prediction_page ();
-        label = gtk_label_new (_("Prediction"));
-        gtk_widget_show (label);
-        gtk_notebook_append_page (GTK_NOTEBOOK (notebook), page, label);
-
-        // Create the 3rd page.
-        page = create_candidates_window_page ();
-        label = gtk_label_new (_("Candidates window"));
-        gtk_widget_show (label);
-        gtk_notebook_append_page (GTK_NOTEBOOK (notebook), page, label);
-
-#if 0
-        // Create the 4th page.
-        page = create_toolbar_page ();
-        label = gtk_label_new (_("Toolbar"));
-        gtk_widget_show (label);
-        gtk_notebook_append_page (GTK_NOTEBOOK (notebook), page, label);
-
-        // Create the 5th page.
-        page = create_dict_page ();
-        label = gtk_label_new (_("Dictionary"));
-        gtk_widget_show (label);
-        gtk_notebook_append_page (GTK_NOTEBOOK (notebook), page, label);
-#endif
-
         // Create the key bind pages.
         page = create_keyboard_page ();
         label = gtk_label_new (_("Key bindings"));
         gtk_widget_show (label);
         gtk_notebook_append_page (GTK_NOTEBOOK (notebook), page, label);
 
+        // Create the prediction page.
+        page = create_prediction_page ();
+        label = gtk_label_new (_("Prediction"));
+        gtk_widget_show (label);
+        gtk_notebook_append_page (GTK_NOTEBOOK (notebook), page, label);
+
+        // Create the candidate page.
+        page = create_candidates_window_page ();
+        label = gtk_label_new (_("Candidates window"));
+        gtk_widget_show (label);
+        gtk_notebook_append_page (GTK_NOTEBOOK (notebook), page, label);
+
+#if 0
+        // Create the toolbar page.
+        page = create_toolbar_page ();
+        label = gtk_label_new (_("Toolbar"));
+        gtk_widget_show (label);
+        gtk_notebook_append_page (GTK_NOTEBOOK (notebook), page, label);
+
+        // Create the dictionary page.
+        page = create_dict_page ();
+        label = gtk_label_new (_("Dictionary"));
+        gtk_widget_show (label);
+        gtk_notebook_append_page (GTK_NOTEBOOK (notebook), page, label);
+#endif
+
         // for preventing enabling left arrow.
         gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), 1);
         gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), 0);
@@ -1364,6 +1520,16 @@ setup_widget_value ()
                                 entry.value.c_str ());
     }
 
+    for (unsigned int i = 0; i < __config_color_common_num; i++) {
+        ColorConfigData &entry = __config_color_common[i];
+        if (entry.widget) {
+            GdkColor color;
+            gdk_color_parse (entry.value.c_str (), &color);
+            gtk_color_button_set_color (GTK_COLOR_BUTTON (entry.widget),
+                                        &color);
+        }
+    }
+
     for (unsigned int j = 0; j < __key_conf_pages_num; ++j) {
         for (unsigned int i = 0; __key_conf_pages[j].data[i].key; ++ i) {
             if (__key_conf_pages[j].data[i].widget) {
@@ -1399,6 +1565,11 @@ load_config (const ConfigPointer &config)
         entry.value = config->read (String (entry.key), entry.value);
     }
 
+    for (unsigned int i = 0; i < __config_color_common_num; i++) {
+        ColorConfigData &entry = __config_color_common[i];
+        entry.value = config->read (String (entry.key), entry.value);
+    }
+
     for (unsigned int j = 0; j < __key_conf_pages_num; ++ j) {
         for (unsigned int i = 0; __key_conf_pages[j].data[i].key; ++ i) {
             __key_conf_pages[j].data[i].value =
@@ -1415,6 +1586,9 @@ load_config (const ConfigPointer &config)
     for (unsigned int i = 0; i < __config_string_common_num; i++)
         __config_string_common[i].changed = false;
 
+    for (unsigned int i = 0; i < __config_color_common_num; i++)
+        __config_color_common[i].changed = false;
+
     for (unsigned int j = 0; j < __key_conf_pages_num; j++) {
         for (unsigned int i = 0; __key_conf_pages[j].data[i].key; ++ i)
             __key_conf_pages[j].data[i].changed = false;
@@ -1443,6 +1617,13 @@ save_config (const ConfigPointer &config)
         entry.changed = false;
     }
 
+    for (unsigned int i = 0; i < __config_color_common_num; i++) {
+        ColorConfigData &entry = __config_color_common[i];
+        if (entry.changed)
+            entry.value = config->write (String (entry.key), entry.value);
+        entry.changed = false;
+    }
+
     for (unsigned int j = 0; j < __key_conf_pages_num; j++) {
         for (unsigned int i = 0; __key_conf_pages[j].data[i].key; ++ i) {
             if (__key_conf_pages[j].data[i].changed)
@@ -1475,6 +1656,16 @@ on_default_toggle_button_toggled (GtkToggleButton *togglebutton,
     }
 }
 
+static void
+on_toggle_button_toggled_set_sensitive (GtkToggleButton *togglebutton,
+                                        gpointer         user_data)
+{
+    GtkWidget *widget = static_cast<GtkWidget*> (user_data);
+
+    gboolean active = gtk_toggle_button_get_active (togglebutton);
+    gtk_widget_set_sensitive (widget, active);
+}
+
 static void
 on_default_editable_changed (GtkEditable *editable,
                              gpointer     user_data)
@@ -1544,6 +1735,27 @@ on_default_combo_changed (GtkEditable *editable,
     }
 }
 
+static void
+on_default_color_button_set (GtkColorButton *colorbutton,
+                             gpointer        user_data)
+{
+    ColorConfigData *entry = static_cast<ColorConfigData*> (user_data);
+
+    if (entry) {
+        GdkColor color;
+        gchar color_str[8];
+        gtk_color_button_get_color (colorbutton, &color);
+        g_snprintf (color_str, G_N_ELEMENTS (color_str),
+                    "#%02X%02X%02X", 
+                    (color.red>>8),
+                    (color.green>>8),
+                    (color.blue>>8));
+        entry->value = String (color_str);
+        entry->changed = true;
+        __have_changed = true;
+    }
+}
+
 static void
 on_key_category_menu_changed (GtkOptionMenu *omenu, gpointer user_data)
 {
@@ -1642,6 +1854,32 @@ on_key_list_view_button_press (GtkWidget *widget, GdkEventButton *event,
 
     return FALSE;
 }
+
+static void
+on_key_list_selection_changed (GtkTreeSelection *selection, gpointer data)
+{
+    GtkTreeModel *model = NULL;
+    GtkTreeIter iter;
+
+    gboolean selected;
+
+    selected = gtk_tree_selection_get_selected (selection, &model, &iter);
+
+    if (__widget_choose_keys_button) {
+        if (selected) {
+            gtk_widget_set_sensitive (__widget_choose_keys_button, true);
+        } else {
+            gtk_widget_set_sensitive (__widget_choose_keys_button, false);
+        }
+    }
+}
+
+static void
+on_choose_keys_button_clicked (GtkWidget *button, gpointer data)
+{
+    GtkTreeView *treeview = GTK_TREE_VIEW (data);
+    key_list_view_popup_key_selection (treeview);
+}
 /*
 vi:ts=4:nowrap:ai:expandtab
 */
